--
-- QWOP-like by knarf <k@b0.cx>
-- 

-- box2D new box shape+body
function box(x, y, w, h, m, r)
   local o = {}
   r = r or 0
   x = x + w/2
   y = y + h/2
   o.body  = love.physics.newBody(world, x, y, m, r)
   o.shape = love.physics.newRectangleShape(o.body, 0, 0, w, h)
   o.color = {math.random(256), math.random(256), math.random(256)}
   return o
end

-- draw box
function dr(s)
   love.graphics.setColor(s.color[1], s.color[2], s.color[3])
   love.graphics.polygon("fill", s.shape:getPoints())
end

function love.load()
   width = 650
   height = 650

   world = love.physics.newWorld(0, 0, width, height)
   world:setGravity(0, 10)
   world:setMeter(64)

   ground = box(0, height-20, width, 20, 0)

   height = height+200
   top = height
   
   torso = box(width/2, height/2, 20, 100, 50, .5)
   front = box(width/2+2.5, height/2+80, 15, 80, 15, .2)
   back  = box(width/2+2.5, height/2+80, 15, 80, 15, .1)
   fcalf = box(width/2+2.5, height/2+150, 15, 50, 10, .1)
   bcalf = box(width/2+2.5, height/2+150, 15, 50, 10, .1)
   
   -- 2 joints for each leg
   ftjoint = love.physics.newRevoluteJoint(front.body, torso.body, 
					   width/2+10, height/2+90)
   btjoint = love.physics.newRevoluteJoint(back.body,  torso.body,
					   width/2+10, height/2+90)

   fljoint = love.physics.newRevoluteJoint(fcalf.body, front.body, 
					   width/2+10, height/2+160)
   bljoint = love.physics.newRevoluteJoint(bcalf.body, back.body,
					   width/2+10, height/2+160)
   height = height-200

   -- suppress some collision
   ftjoint:setCollideConnected(false)
   btjoint:setCollideConnected(false)
   fljoint:setCollideConnected(false)
   bljoint:setCollideConnected(false)
   
   ground.shape:setCategory(2)

   front.shape:setCategory(1)
   back.shape:setCategory(1)
   fcalf.shape:setCategory(1)
   bcalf.shape:setCategory(1)
   
   front.shape:setMask(1)
   back.shape:setMask(1)
   fcalf.shape:setMask(1)
   bcalf.shape:setMask(1)
   
   -- limit rotation of legs
   ftjoint:setLimits(-math.pi/2, math.pi/2)
   btjoint:setLimits(-math.pi/2, math.pi/2)
   fljoint:setLimits(-math.pi, 0)
   bljoint:setLimits(-math.pi, 0)

   ftjoint:setLimitsEnabled(true)
   btjoint:setLimitsEnabled(true)
   fljoint:setLimitsEnabled(true)
   bljoint:setLimitsEnabled(true)

   love.graphics.setMode(width, height, false, true, 0)
end

function maxheight(s)
   local p = {s.shape:getPoints()}
   local max = p[2]
   
   for i = 2, #p, 2 do
      if p[i] < max then 
	 max = p[i]
      end
   end
   return max   
end

function reset()
   torso.body:setX(width/2)
   torso.body:setY(height/2+200)
   top = height
end

function love.quit()
   return false
end

function love.keypressed(key, unicode)
   local a = 1000 -- thighs
   local b = 400  -- calves

   if key == 'escape' then
      love.event.push("q")

   elseif key == 'r' then
      reset()

   elseif key == 'q' then
      front.body:applyTorque(a)
      back.body:applyTorque(-a)
   
   elseif key == 'w' then
      front.body:applyTorque(-a)
      back.body:applyTorque(a)

   elseif key == 'o' then
      fcalf.body:applyTorque(-b)
      bcalf.body:applyTorque(b)

   elseif key == 'p' then
      fcalf.body:applyTorque(-b)
      bcalf.body:applyTorque(b)
   end
   
end


function love.update(dt)
   world:update(dt)
   top = math.min(top, maxheight(torso),
		  maxheight(front),
		  maxheight(back),
		  maxheight(fcalf),
		  maxheight(bcalf))
end


function love.draw()
   dr(ground)
   dr(torso)
   dr(back)
   dr(front)
   dr(bcalf)
   dr(fcalf)

   -- local scale = .5+(height-top)/height
   local scale = math.sin(4*love.timer.getTime())*.1+1
   local str = string.format("TOP %.1f", height-top)

   love.graphics.setColor(255, 255, 255)
   love.graphics.print("QWOP: high jump style!\n"..
		       "Use Q, W, O and P (in qwerty "..
		       "layout) to control the thighs/calves.\n"..
		       "ESC to quit, R to reset.\n"..
		       "Good luck!", 30, 30)

   love.graphics.line(0, top, width, top)
   love.graphics.print(str, width/2-30*scale, top, 0, scale, scale)
   
end
